#pragma once

#include <dshow.h>
#include <streams.h>
#include <mpconfig.h>
#include <atlbase.h>

class movieplayer
{
private:
	CComPtr<IGraphBuilder> m_graph;
	CComPtr<IMediaControl> m_control;
	CComPtr<IMediaSeeking> m_seeking;
	CComPtr<IMediaEvent>   m_event;

	CComPtr<IDDrawExclModeVideo> m_excl_video;

	ATL::CStringW m_filename;
	RECT m_videorect;

	IBaseFilter *FilterFindByCLSIDs(IFilterGraph *graph, int count, CLSID *array)
	{
		CComPtr<IEnumFilters> flt_enum;
		graph->EnumFilters(&flt_enum);
		IBaseFilter *filter = NULL;
		while(S_OK == flt_enum->Next(1, &filter, NULL)) {
			CLSID clsid;
			if(SUCCEEDED(filter->GetClassID(&clsid)))
				for(int i = 0; i < count; ++i)
					if(!memcmp(&clsid, &array[i], sizeof(CLSID)))
						return filter;
			filter->Release();
		}
		return NULL;
	}

	IPin * GetInPin(IBaseFilter *filter, int Num)
	{
		CComPtr<IEnumPins> pin_enum;
		filter->EnumPins(&pin_enum);

		IPin *pin;
		while(S_OK == pin_enum->Next(1, &pin, NULL)) {
			PIN_DIRECTION direction;
			pin->QueryDirection(&direction);
			if(direction == PINDIR_INPUT && !Num--)
				return pin;
			pin->Release();
		}
		return NULL;
	}
public:
	movieplayer() {
		m_videorect.left = m_videorect.top = 0;
		m_videorect.right = m_videorect.bottom = 0;
	}

	bool BuildGraph(LPDIRECTDRAW dd, LPDIRECTDRAWSURFACE surf)
	{
		m_excl_video.Attach(NULL);
		m_event.Attach(NULL);
		m_seeking.Attach(NULL);
		m_control.Attach(NULL);
		m_graph.Attach(NULL);

		CComPtr<IGraphBuilder> graph;
		if(FAILED(graph.CoCreateInstance(CLSID_FilterGraph))) {
			OutputDebugString("CoCreateInstance");
			return false;
		}

		CComPtr<IBaseFilter> overlay;
		if(FAILED(overlay.CoCreateInstance(CLSID_OverlayMixer))) {
			OutputDebugString("CoCreateInstance");
			return false;
		}

		CComQIPtr<IDDrawExclModeVideo> excl_video(overlay);
		if(FAILED(excl_video->SetDDrawObject(dd)))
		{
			OutputDebugString("SetDDrawObject");
			return false;
		}
		if(FAILED(excl_video->SetDDrawSurface(surf)))
		{
			OutputDebugString("SetDDrawSurface");
			return false;
		}
		if(FAILED(excl_video->SetDrawParameters(NULL, &m_videorect)))
		{
			OutputDebugString("SetDrawParameters");
			return false;
		}

		if(FAILED(graph->AddFilter(overlay, L"Overlay Mixer"))) {
			OutputDebugString("AddFilter");
			return false;
		}
		if(FAILED(graph->RenderFile(m_filename, NULL))) {
			OutputDebugString("RenderFile");
			return false;
		}

		CComPtr<IPin> overlay_in;
		CComPtr<IPin> decoder_out;
		overlay_in.Attach(GetInPin(overlay, 0));
		if(FAILED(overlay_in->ConnectedTo(&decoder_out)) || decoder_out == NULL) {
			CComPtr<IBaseFilter> renderer;
			CLSID clsids[] = { CLSID_VideoMixingRenderer, CLSID_VideoRendererDefault, CLSID_VideoRenderer };
			renderer.Attach(FilterFindByCLSIDs(graph, 3, clsids));
			if(renderer == NULL) {
				OutputDebugString("FilterFindByCLSIDs");
				return false;
			}
			CComPtr<IPin> renderer_in;
			renderer_in.Attach(GetInPin(renderer, 0));
			if(FAILED(renderer_in->ConnectedTo(&decoder_out))) {
				OutputDebugString("ConnectedTo2");
				return false;
			}
			if(FAILED(graph->Disconnect(renderer_in))) {
				OutputDebugString("Disconnect");
				return false;
			}
			if(FAILED(graph->Disconnect(decoder_out))) {
				OutputDebugString("Disconnect");
				return false;
			}
			if(FAILED(graph->Connect(decoder_out, overlay_in))) {
				OutputDebugString("Connect");
				return false;
			}
			if(FAILED(graph->RemoveFilter(renderer))) {
				OutputDebugString("RemoveFilter");
				return false;
			}
			OutputDebugString("Hage");
		}

		CComQIPtr<IMixerPinConfig, &IID_IMixerPinConfig> pinconf(overlay_in);
		if(!pinconf) {
			OutputDebugString("QueryInterface");
			return false;
		}
		COLORKEY newkey = { CK_RGB, 0, 0, 0 };
		if(FAILED(pinconf->SetColorKey(&newkey)))
			return false;

		m_excl_video.Attach(excl_video.Detach());
		m_graph.Attach(graph.Detach());
		m_graph.QueryInterface(&m_control);
		m_graph.QueryInterface(&m_seeking);
		m_graph.QueryInterface(&m_event);
		return true;
	}

	void SetFilePath(LPCWSTR filename)
	{
		m_filename = filename;
	}

	void SetVideoRect(LPRECT rect)
	{
		memcpy(&m_videorect, rect, sizeof(RECT));
		if(m_excl_video)
			m_excl_video->SetDrawParameters(NULL, &m_videorect);
	}

	bool Play()
	{
		if(m_graph == NULL) return false;
		return m_control->Run() == S_OK;
	}

	bool Stop()
	{
		if(m_graph == NULL) return false;
		return m_control->StopWhenReady() == S_OK;
	}

	bool SetPosition(LONGLONG pos, DWORD flag)
	{
		if(m_graph == NULL) return false;
		flag = 
			(flag == 0) ? AM_SEEKING_AbsolutePositioning :
			(flag == 1) ? AM_SEEKING_RelativePositioning :
			AM_SEEKING_NoPositioning;
		return m_seeking->SetPositions(&pos, flag, NULL, AM_SEEKING_NoPositioning) == S_OK;
	}
	bool IsStoped()
	{
		if(m_graph == NULL) return false;
		long evcode;
		return m_event->WaitForCompletion(0, &evcode) != E_ABORT;
	}
};
