#pragma once

#include <stdio.h>
#include <vorbis/vorbisfile.h>

class ovfile {
private:
	bool opened;
	OggVorbis_File vf;

public:
	ovfile() : opened(false) {}

	~ovfile() {
		if(opened) ov_clear(&vf);
	}

	bool open(const char *name, char *initial, long ibytes) {
		FILE* fp = fopen(name, "rb");
		if(!fp) return false;

		if(ov_open(fp, &vf, initial, ibytes) < 0) {
			fclose(fp);
			fp = NULL;
			return false;
		}

		opened = true;
		return true;
	}

	vorbis_info * info() {
		if(!opened) return NULL;
		return ov_info(&vf, -1);
	}

	ogg_int64_t pcm_total() {
		if(!opened) return NULL;
		return ov_pcm_total(&vf, -1);
	}

	long read(char *buf, long len) {
		if(!opened) return NULL;

		int bitstream;
		long total = 0;
		long readlen;

		while(len > total) {
			readlen = ov_read(
				&vf,
				buf, len - total,
				0, 16 >> 3, 1, &bitstream);
			if (readlen == OV_HOLE ||
				readlen == OV_EBADLINK ||
				readlen == 0) break;
			total += readlen;
			buf   += readlen;
		}

		return total;
	}
};
